# FELDMAN PERFORMANCE FUNCTION
#
# by Steven Yi
# email: stevenyi@csounds.com
# web: http://www.csounds.com/stevenyi
#
# performs music in the style of Morton Feldman's early graphic scores
# this file has been extracted from my feldman2.blue work file 
# for use with the blue composition environment.
#
# some comments are geared towards the python newbie
#
# released in python standalone form on Jan 26th, 2003
#
# ======================================================================


# Below is a python import statement
# It allows for using functions from the "random" library in this script

import random

#############################################################
# FUNCTION FOR PERFORMING FELDMAN GRAPHIC SCORE - VERSION 2 #
#############################################################

# The performFeldmanGraphicScore function takes in an "hmllist" (High-Middle-Low list, 
# the score in the "My Score" soundObject) and "performs" the score in the duration given.

# This version of the function takes in a new parameter, iNum, which tells the function for
# what Csound instrument to generate notes for.  This was done so another "performance" of
# the score on another instrument could be done.

def performFeldmanGraphicScore(hmlList, duration, iNum):
	returnString = ""

	measureLength = float(duration) / len(hmlList)

	hi_top 		= 3840
	hi_bottom 		= 960
	mid_top		= 960
	mid_bottom 	= 240
	low_top		= 240
	low_bottom		= 60
	noteDur		= float(measureLength) / 2

	for measure in range(0, len(hmlList)):
		hml = hmlList[measure]
		for i in range(0, hml[0]):
			start = (random.random() * measureLength) + (measure * measureLength)
			duration = noteDur
			pch = (random.random() * (hi_top - hi_bottom)) + hi_bottom
			amp = 70
			space = 0
			env = 0

			returnString += "i" + str(iNum) + " " + str(start) + "\t" + str(duration) + "\t" + str(pch) + "\t" + str(pch) + "\t" + str(amp) + "\t" + str(space) + "\t" + str(env) + "\n"


		for i in range(0, hml[1]):
			start = (random.random() * measureLength) + (measure * measureLength)
			duration = noteDur
			pch = (random.random() * (mid_top - mid_bottom)) + mid_bottom
			amp = 70
			space = 0
			env = 0

			returnString += "i" + str(iNum) + " " + str(start) + "\t" + str(duration) + "\t" + str(pch) + "\t" + str(pch) + "\t" + str(amp) + "\t" + str(space) + "\t" + str(env) + "\n"

		for i in range(0, hml[2]):
			start = (random.random() * measureLength) + (measure * measureLength)
			duration = noteDur
			pch = (random.random() * (low_top - low_bottom)) + low_bottom
			amp = 70
			space = 0
			env = 0

			returnString += "i" + str(iNum) + " " + str(start) + "\t" + str(duration) + "\t" + str(pch) + "\t" + str(pch) + "\t" + str(amp) + "\t" + str(space) + "\t" + str(env) + "\n"

	return returnString



###################################################
# DEFINING SCORE FOR FELDMAN PERFORMANCE FUNCTION #
###################################################

# The score here is list (my measures).  It starts off with a blank "score", then add measures to it.
# Each measure is itself a list of three numbers, pertaining to the number of "high", "middle", and
# "low" notes to produce for that measure.

measures = []

measures.append([2,0,0])	# this can be read as "two high notes, no middle notes, no low notes"
measures.append([0,0,0])	# this can be read as "no notes for this measure"
measures.append([0,1,2])
measures.append([1,0,5])
measures.append([2,0,0])
measures.append([0,0,0])
measures.append([0,2,0])
measures.append([1,0,4])
measures.append([0,0,0])
measures.append([0,0,0])
measures.append([0,3,2])
measures.append([0,2,3])
measures.append([0,0,0])
measures.append([0,0,0])
measures.append([1,1,1])



###########################################################
# PERFORMING THE SCORE USING FELDMAN PERFORMANCE FUNCTION #
###########################################################

# below is the actual "performance" of the score, which uses my "measures" list
# defined in the "My Score" block. 

score = performFeldmanGraphicScore(measures, 45, 3)

# a second performer performing the same score

score += performFeldmanGraphicScore(measures, 42, 4)


# Sinewave FTable needed for Simple FM Instruments
ftable = "f1 0 65537 10 1  ; sine wave\n"

# print score to screen
#print (score)

# write score to file
f = open("feldman.sco", "w")
f.write(ftable)
f.write(score)
f.close()